/*
 *  mTabChat - Plugin dla komunikatora Tlen.pl dodajcy zakadki do okna 
 *             czata/konferencji, podobne do tych z okna rozmowy.
 *
 *  Copyright (C) 2006 Marcin "MalCom" Malich <malcom@o2.pl>
 *  http://malcom.xorg.pl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// mTabChat.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "AggressiveOptimize.h"
#include "resource.h"
#include "chatwindow.h"
#include "mTabChat.h"
#include "options.h"
#include "protocol.h"


HINSTANCE hInst;
TLENPLUGINFUNCTIONS* tlen_functions;
HWND hOkno = NULL, hTab = NULL, hSubject = NULL;

HBRUSH hbrBkColor, hbrBkLight;
UINT bkColor = RGB(228,226,212);	// kolor tla dla wszystkich elementow
UINT bkLight = RGB(255,250,184);	// kolor light, dla zakaldek podswietlanych
HFONT hFont;
HBITMAP hBitmap1, hBitmap2;

int display = 0;	// wyswietlana zakladka
vec_tab tab;


TLENPLUGININFO pluginInfo = {
	sizeof(TLENPLUGININFO),
	"mTabChat",
	PLUGIN_API_VERSION,
	MAKE_DWORD_VERSION(0,0,0,7),
	"Plugin dodaje zakadki do okna czata/konferencji, podobnie jak zakadki w oknie rozmowy.",
	" 2006 MalCom",
	"MalCom",
	"malcom@o2.pl",
	"http://malcom.xorg.pl",
	0, 0, 0, 0
};


extern "C" __declspec(dllexport) TLENPLUGININFO* GetPluginInfo(DWORD TlenVersion);
extern "C" __declspec(dllexport) int LoadPlugin(TLENPLUGINFUNCTIONS* tlen_functions);
extern "C" __declspec(dllexport) int UnloadPlugin(void);


// ustaw okno czata / skoryguj pozycje i rozmiary
void SetTabPosition(int idx) {
	RECT rc;
	GetClientRect(hTab, &rc);
	TabCtrl_AdjustRect(hTab, false, &rc);
	MoveWindow(tab[idx].Handle, rc.left, rc.top, rc.right-rc.left+2, rc.bottom-rc.top+2, TRUE);
	ShowWindow(tab[idx].Handle, SW_SHOW);
	std::string title = tab[idx].Name + " [mTabChat]";
	SetWindowText(hOkno, title.c_str());
	SetFocus(tab[idx].hMemo);
}

// pobierz/ustaw nazwy okien czata
void SetTabTitle() {
	int j = tab.size();
	for (int i=0; i<j; i++) {
		char buf[256] = {0};
		GetWindowText(tab[i].Handle, buf, sizeof(buf)-1);
		tab[i].Name = buf;
		strcat(buf, "   ");		// 3 spacje, aby bylo miejsce na button :P
				
		TCITEM item;
			item.mask = TCIF_TEXT;
			item.pszText = buf;
		TabCtrl_SetItem(hTab, i, &item);
	}
	SetTabPosition(display);
}

// highLight dla zakladki id / m=1 light / m=0 off
void LightItem(int idx, int m) {
	if (m == 1) {
		// jesli nie ma id zakladki i nie jest aktywna to dodaj i wymus redraw
		// jak jest olej, po co obciazac system odmalwoaniem
		if (!tab[idx].Light && idx != TabCtrl_GetCurSel(hTab)) {
			tab[idx].Light = true;
			InvalidateRect(hTab, NULL, TRUE);
		}
		if (GetForegroundWindow() != hOkno)
			// okno nie aktywne (nie na wierzchu) to daj migotanie tytulu na pasku zadan
			FlashWindow(hOkno, true);
	} else {
		tab[idx].Light = false;
		InvalidateRect(hTab, NULL, TRUE);
	}
}

// subcclasing tab do przechwycenia WM_MOUSEMOVE
LRESULT CALLBACK TabNevWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {
	if (message == WM_MOUSEMOVE) {
		// jesli najechano na jedne z buttonow zakladek to podmien obrazki
		// w przeciwnym razie namaluj stary
		POINT pt;
		pt.x = GET_X_LPARAM(lParam);
		pt.y = GET_Y_LPARAM(lParam);
		int j = tab.size();
		for (int i=0; i<j; i++) {
			HDC hDc = GetDC(hOkno);
			HDC chDc = CreateCompatibleDC(hDc);
			if (PtInRect(&tab[i].RcIcon, pt)) {
				SelectObject(chDc, hBitmap2);
				TransparentBlt(hDc, tab[i].RcIcon.left, tab[i].RcIcon.top , 9, 9, chDc, 0, 0, 9, 9, RGB(255,0,255));
			} else {
				SelectObject(chDc, hBitmap1);
				TransparentBlt(hDc, tab[i].RcIcon.left, tab[i].RcIcon.top , 9, 9, chDc, 0, 0, 9, 9, RGB(255,0,255));
			}
			DeleteDC(chDc);
		}
	}
	return CallWindowProc((WNDPROC) GetWindowLong(hwnd, GWL_USERDATA), hwnd, message, wParam, lParam);	//	return 0;
}


// procedura glownego okna pluginu
LRESULT CALLBACK DialogProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {

	switch (message) {

		case WM_INITDIALOG: {
			hbrBkColor = CreateSolidBrush(bkColor);		// kolor tla dla wszystkich elementow
			hbrBkLight = CreateSolidBrush(bkLight);		// kolor light, dla zakladek podswietlanych

			// bitmapy do buttonow zakladek
			hBitmap1 = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP1));
			hBitmap2 = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP2));

			// ikona ze zrodel, bo te tlenowe to ujnia :/
			SendMessage(hwnd, WM_SETICON, ICON_BIG, (LONG)LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON)));

			hTab = GetDlgItem(hwnd, IDC_TAB);
			SetClassLong(hTab,GCL_HBRBACKGROUND, (LONG) hbrBkColor);
			// subcclasing tab do przechwycenia WM_MOUSEMOVE
			SetWindowLong(hTab, GWL_USERDATA, SetWindowLong(hTab, GWL_WNDPROC, (LONG) TabNevWndProc));

			ShowWindow(hwnd, SW_SHOW);
			return true;
		}

		case WM_DRAWITEM: {
			char buf[256]={0};

			LPDRAWITEMSTRUCT lpdis;
			lpdis = (LPDRAWITEMSTRUCT) lParam;	// item drawing information

			if (hTab == lpdis->hwndItem) {		// is this the tab control?

				TCITEM item;
					item.mask = TCIF_TEXT;
					item.pszText = buf;
					item.cchTextMax = sizeof(buf);
				TabCtrl_GetItem(hTab, lpdis->itemID, &item);

				lpdis->rcItem.bottom +=2;			// pozbycie sie paska tla standard. w niekatywnej ;)

				if (tab[lpdis->itemID].Light) {		// podswietlony tab
					FillRect(lpdis->hDC, &lpdis->rcItem, hbrBkLight);
					SetBkColor(lpdis->hDC, bkLight);
				} else {
					FillRect(lpdis->hDC, &lpdis->rcItem, hbrBkColor);
					SetBkColor(lpdis->hDC, bkColor);
				}

				int left = lpdis->rcItem.right-10;	// button ma 9 + 2 odstepu od prawej
				int top = lpdis->rcItem.top+5;
				
				// jesli aktywna zakladka to o 3px jeszcze bardziej w prawo ;)
				if (TabCtrl_GetCurSel(hTab) == lpdis->itemID) {
					lpdis->rcItem.left += 3;
					left -= 2;
				}

				TextOut(lpdis->hDC, lpdis->rcItem.left+3, lpdis->rcItem.top+3, item.pszText, lstrlen(item.pszText));

				// butony zamykania
				HDC hDc = GetDC(hOkno);
				HDC chDc = CreateCompatibleDC(hDc);
				SelectObject(chDc, hBitmap1);
				TransparentBlt(hDc, left, top , 9, 9, chDc, 0, 0, 9, 9, RGB(255,0,255));
				DeleteDC(chDc);
				
				tab[lpdis->itemID].RcIcon.left = left;
				tab[lpdis->itemID].RcIcon.top = top;
				tab[lpdis->itemID].RcIcon.right = left+9;
				tab[lpdis->itemID].RcIcon.bottom = top+9;
			}

			break;
		}

		case WM_SIZING: {
			RECT* rc = (LPRECT) lParam;
			int cy = rc->bottom - rc->top;
			int cx = rc->right - rc->left;
			if (cy <= MIN_CY) rc->bottom = rc->top + MIN_CY;
			if (cx <= MIN_CX) rc->right = rc->left + MIN_CX;
			return 1;
		}

		case WM_WINDOWPOSCHANGED: {
			RECT rc;
			GetClientRect(hwnd, &rc);
			MoveWindow(hTab, 1, 1, rc.right-rc.left-2, rc.bottom-rc.top-2, TRUE);
			if (tab.size() > 0)
				SetTabPosition(display);
			config.width = rc.right - rc.left;
			config.height = rc.bottom - rc.top;
			break;
		}			

		case WM_CTLCOLORDLG: {
			return (LRESULT) hbrBkColor;
		}

//		case WM_PAINT:
//			break;

		case WM_DESTROY: {
			display = 0;
			hOkno = NULL;
			DeleteObject(hFont);
			DeleteObject(hBitmap1);
			DeleteObject(hBitmap2);
			break;
		}

		case WM_CLOSE: {
			char buffer[100];	char* buff;		char* buf;
			int ret;
			if (config.alert) {
				int k = TabCtrl_GetItemCount(hTab);
				if (k == 1) {
					buff = "u";
					buf = "go";
				} else {
					buff = "ach";
					buf = "je wszystkie";
				}
				sprintf(buffer,"Przebywasz w %d pokoj%s.\nCzy napewno chcesz %s opuci?", k, buff, buf);
				ret = MessageBox(hOkno, buffer, "Wyjcie...", MB_YESNO | MB_ICONEXCLAMATION);
			} else {
				ret = IDYES;
			}

			if (ret == IDYES) {
				// zamknij wszystkie okna/zakladki
				// nalezy od gory, bo przy zamknieciu okna index-1 dla tabow
				for (int i=tab.size(); i>=0; i--)
					SendMessage(tab[i].Handle, WM_CLOSE, NULL, NULL);
				DestroyWindow(hOkno);
			}
	
			break;
		}

//		case WM_COMMAND:
//			break;

		case WM_NOTIFY: {

			LPNMHDR lpnmh;
			lpnmh = (LPNMHDR) lParam;

			if (lpnmh->code == TCN_SELCHANGE) {				// zmiana zakladki
				ShowWindow(tab[display].Handle, SW_HIDE);
				display = TabCtrl_GetCurSel(hTab);
				LightItem(display, 0);
				SetTabPosition(display);

			} else if (lpnmh->code == NM_CLICK) {
				// kliknieto na zakaldke, pobierz wspolrzedne kursora
				// jesli sa na polu buttona to zamknij okno
				POINT pt;
				GetCursorPos(&pt);
				ScreenToClient(hwnd, &pt);
				int j = tab.size();
				for (int i=0; i<j; i++) {
					if (PtInRect(&tab[i].RcIcon, pt))
						SendMessage(tab[i].Handle, WM_CLOSE, NULL, NULL);
				}
			}
			break;
		}
	}
	return 0;
}


BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
	hInst = hinstDLL;
	return true;
}

__declspec(dllexport) TLENPLUGININFO* GetPluginInfo(DWORD tlenVersion) {
	return &pluginInfo;
}

__declspec(dllexport) int LoadPlugin(TLENPLUGINFUNCTIONS* functions) {
	tlen_functions = functions;

	LoadConfig();
	AddHook();

	chat_hook = tlen_functions->HookTlenEvent(hInst, TLEN_CHATWINDOW_EVENT, ChatWindowEvent);

	return 0;
}

__declspec(dllexport) int UnloadPlugin(void) {

	DefaultWindows();
	SaveConfig();
	DelHook();

	tlen_functions->UnhookTlenEvent(hInst, chat_hook);
	if (hOkno) DestroyWindow(hOkno);

	return 0;
}
